#!/bin/bash
BASE_DIR=$(pwd)
CURL_DIR=${BASE_DIR}'/src/lib/curl-7.15.1'
WWW_DIR=${BASE_DIR}'/src/lib/w3c-libwww-5.4.0'
XMLRPC_DIR=${BASE_DIR}'/src/lib/xmlrpc-c-1.03.11'
CURL_CONFIG=$(which curl-config 2>/dev/null)
WWW_CONFIG=$(which libwww-config 2>/dev/null)

# If curl is not built, build it now.
if [ ! -f ${CURL_DIR}'/lib/.libs/libcurl.so' ]; then
	echo
	echo "<!------------------------!>"
	echo "Building the curl libraries."
	echo "<!------------------------!>"
	echo
	sleep 3
	cd ${CURL_DIR}
	./configure
	make

	# If curl is not installed, install it
	if [ ! -n "$CURL_CONFIG" ]; then
		echo
		echo "<!--------------------------!>"
		echo "Installing the curl libraries."
		echo "<!--------------------------!>"
		echo
		echo "Please enter your password for sudo access."
		sudo make install
	fi

	# Copy curl libs to the lib directory
	cp ${CURL_DIR}'/lib/.libs/'*.so.3 ${BASE_DIR}'/lib'
fi

# If libwww is not built, build it now.
if [ ! -f ${WWW_DIR}'/Library/src/.libs/libwwwcore.so' ]; then
	echo
	echo "<!------------------------!>"
	echo "Building the www libraries."
	echo "<!------------------------!>"
	echo
	sleep 3
	cd ${WWW_DIR}
	./configure --with-expat --with-md5 --with-ssl --with-zlib
	make

	# If libwww is not installed, install it
	if [ ! -n "$WWW_CONFIG" ]; then
		echo
		echo "<!-------------------------!>"
		echo "Installing the www libraries."
		echo "<!-------------------------!>"
		echo
		echo "Please enter your password for sudo access."
		sudo make install
	fi

	# Copy www libs to the lib directory
	cp ${WWW_DIR}'/Library/src/.libs/'*.so.0 ${BASE_DIR}'/lib'
	cp ${WWW_DIR}'/modules/expat/xmltok/.libs/'*.so.0 ${BASE_DIR}'/lib'
	cp ${WWW_DIR}'/modules/expat/xmlparse/.libs/'*.so.0 ${BASE_DIR}'/lib'
	cp ${WWW_DIR}'/modules/md5/.libs/'*.so.0 ${BASE_DIR}'/lib'
fi

# If xmlrpc-c is not built, build it now.
if [ ! -f ${XMLRPC_DIR}'/src/cpp/libxmlrpc++.a' ]; then
	echo
	echo "<!------------------------!>"
	echo "     Building xmlrpc-c."
	echo "<!------------------------!>"
	echo
	sleep 3
	cd ${XMLRPC_DIR}
	./configure --enable-curl-client --enable-libwww-client --enable-abyss-server --enable-cplusplus --enable-abyss-threads
	make

	# Copy xmlrpc-c libs to the lib directory
	cp ${XMLRPC_DIR}'/src/.libs/'*.so.3 ${BASE_DIR}'/lib'
	cp ${XMLRPC_DIR}'/lib/expat/xmltok/.libs/'*.so.3 ${BASE_DIR}'/lib'
	cp ${XMLRPC_DIR}'/lib/expat/xmlparse/.libs/'*.so.3 ${BASE_DIR}'/lib'
	cp ${XMLRPC_DIR}'/lib/abyss/src/.libs/'*.so.3 ${BASE_DIR}'/lib'
fi

# Now build the example programs.
echo
echo "<!------------------------!>"
echo " Building PRC applications."
echo "<!------------------------!>"
echo
sleep 1
cd $BASE_DIR
make

rm -rf client server
mkdir -p client/bin server/bin
cp -R lib log client
cp -R lib log server 
cp bin/sci_app client/bin
cp scripts/run_sci_app.sh client
cp bin/work_gen server/bin
cp scripts/run_work_gen.sh server
cp bin/validator server/bin
cp scripts/run_validator.sh server

echo "Build Complete."
