// Local Includes
#include "common/common.hpp"
#include "common/server.hpp"
#include "rpc_handler.hpp"
#include "sci_app_client.hpp"
#include "sci_data_proc.hpp"

using std::vector;

namespace prc {

/**
 * This method is called when the sciapp.computeResult RPC is handled.
 * @param paramList List of RPC parameters.
 * @param retvalP Pointer to the return value of the method.
 */
void ComputeResultHandler::execute(xmlrpc_c::paramList const& paramList, xmlrpc_c::value* const retvalP) {
	// Enable checkpointing.
	ScienceApplicationClient::instance()->setCheckpointing(true);

	// Get the first parameter to the RPC.
	vector<byte> workUnit = paramList.getBytestring(0);

	// Compute the result for the work unit.
	ScienceDataProcessor proc;
	vector<byte> resultData = proc.run(workUnit);

	// Convert the result data byte array to an xmlrpc_c::value_bytestring.
	xmlrpc_c::value_bytestring resultBytes(resultData);

	// Add the XML-RPC byte string to an XML-RPC array, 
	// which is the format the client is expecting it to be in.
	vector<xmlrpc_c::value> resultVector;
	resultVector.push_back(resultBytes);

	// Set the return value.
	*retvalP = xmlrpc_c::value_array(resultVector);
}

/**
 * This method is called when the sciapp.computeSpotCheckResult RPC is handled.
 * @param paramList List of RPC parameters.
 * @param retvalP Pointer to the return value of the method.
 */
void ComputeSpotCheckResultHandler::execute(xmlrpc_c::paramList const& paramList, xmlrpc_c::value* const retvalP) {
	// Disable checkpointing.
	ScienceApplicationClient::instance()->setCheckpointing(false);

	// Get the first parameter to the RPC.
	vector<byte> workUnit = paramList.getBytestring(0);

	// Compute the result for the work unit.
	ScienceDataProcessor proc;
	vector<byte> resultData = proc.run(workUnit);

	// Convert the result data byte array to an xmlrpc_c::value_bytestring.
	xmlrpc_c::value_bytestring resultBytes(resultData);

	// Add the XML-RPC byte string to an XML-RPC array, 
	// which is the format the client is expecting it to be in.
	vector<xmlrpc_c::value> resultVector;
	resultVector.push_back(resultBytes);

	// Set the return value.
	*retvalP = xmlrpc_c::value_array(resultVector);
}

/**
 * This method is called when the sciapp.shutdown RPC is handled.
 * @param paramList List of RPC parameters.
 * @param retvalP Pointer to the return value of the method.
 */
void ShutdownHandler::execute(xmlrpc_c::paramList const& paramList, xmlrpc_c::value* const retvalP) {
	Server::instance()->shutdown();
	*retvalP = xmlrpc_c::value_boolean(true);
}

} // end namespace prc
