#ifndef _WORK_GEN_CLIENT_HPP
#define _WORK_GEN_CLIENT_HPP

// System includes
#include <cstring>
#include <ctime>
#include <vector>

// Local includes
#include "common/client.hpp"
#include "common/common.hpp"

// Declare which objects from std we're using.
using std::string;
using std::time_t;
using std::vector;

namespace prc {

// This struct stores all work unit data.
struct WorkUnit {
	std::string workUnitID;
	std::vector<byte> data;
	std::time_t creationDate;
	int points;
	int priority;
};

/**
 * The WorkGeneratorClient is an XML-RPC client with 
 * added functionality that is needed by the work unit generator.
 * It has methods for sending new work units to the server, retrieving 
 * the last work unit that was added, and retrieving the number of new
 * (ingress) work units that the server has.
 */
class WorkGeneratorClient : public Client {
	private:
		static WorkGeneratorClient* clientInstance;
		static const string GET_LAST_WU_RPC_NAME;
		static const string GET_NUM_WU_RPC_NAME;
		static const string SEND_WU_RPC_NAME;

		/**
 		 * Private constructor because this class is a singleton.
		 * Note: The default server address is 127.0.0.1:2080, 
		 * but it is possible to run the work unit generator on a 
		 * different computer from the project server, so this 
		 * address can be changed by specifying a new address in 
		 * the constructor or using prc::Client::setServerAddress(string);
		 */
		WorkGeneratorClient(string addr = "http://127.0.0.1:2080/RPC2") : Client(addr) { }
		size_t sendWorkUnit(xmlrpc_c::paramList&, const std::string&);
	public:
		static WorkGeneratorClient* instance();
		WorkUnit getLastWorkUnit();
		size_t sendWorkUnit(const string,const vector<byte>,int,int,double);
		size_t sendWorkUnit(const string,const vector<byte>,int,int);
		size_t sendWorkUnit(const vector<byte>,int,int,double);
		size_t sendWorkUnit(const vector<byte>,int,int);
		size_t sendWorkUnit(const vector<byte>);
		size_t getNumWorkUnits();
};

} // end namespace prc

#endif // _WORK_GEN_CLIENT_HPP
