package edu.wpi.prc.example.java.common;

/**
 * RPCHandlerExceptions are thrown by an RPC client 
 * when the RPC handler being called on a remote host
 * throws an Exception while executing an RPC.  These 
 * Exceptions do not indicate connection errors or 
 * XML-RPC errors.  They are used only when 
 * component-specific RPC handling code throws an
 * Exception.
 * @author James Baldassari
 *
 */
public class RPCHandlerException extends Exception {
	public static final long serialVersionUID = 0;
	private String message = "RPCHandlerException";
	
	/**
	 * Default constructor.
	 */
	public RPCHandlerException() {
		
	}
	
	/**
	 * Constructor specifying an error message.
	 * @param message The error message.
	 */
	public RPCHandlerException(String message) {
		this.message += ": " + message;
	}
	
	/**
	 * Returns a String representing this Exception.
	 * @return The error string.
	 */
	public String toString() {
		return message;
	}
}
