/**
 * This is the client class definition.
 * @author James Baldassari (jdb@cs.wpi.edu)
 */

#ifndef _CLIENT_HPP
#define _CLIENT_HPP

// XML-RPC includes
#include <xmlrpc-c/base.hpp>
#include <xmlrpc-c/client_simple.hpp>

// Declare which objects from std we're using.
using std::string;

// Define Client to be in the prc namespace
namespace prc {

/**
 * The Client class is responsible for initializing XML-RPC
 * and making all XML-RPC requests.  Note: methods are 
 * implemented/documented in client.cpp.
 */
class Client {
	private:
		static const int RETRY_DELAY;
		string serverAddr;		
		xmlrpc_c::clientXmlTransport* xmlrpcTransport;
		xmlrpc_c::client_xml* xmlrpcClient;
		xmlrpc_c::carriageParm_libwww0* cParm;
		void initializeXMLRPCClient();
	public:
		Client(string);
		~Client();
		void setServerAddress(string);
		xmlrpc_c::value executeRPC(xmlrpc_c::rpcPtr&);
		xmlrpc_c::value executeRPCUntilSuccess(xmlrpc_c::rpcPtr&,const string&);
};

} // end namespace prc

#endif // #define _CLIENT_HPP
