// System Includes
#include <iostream>
#include <math.h>
#include <pthread.h>
#include <sched.h>
#include <vector>

// Local Includes
#include "common/common.hpp"
#include "sci_app_client.hpp"
#include "sci_data_proc.hpp"

using std::cout;
using std::endl;
using std::string;
using std::vector;

namespace prc {
/**
 * This method will be called by the RPC handlers.
 * The run method should execute the project-specific 
 * science algorithm on the given work unit and produce 
 * a result in the form of a vector<byte>.  The byte 
 * vector should be passed back in the return value 
 * of this method.
 * NOTE: A "byte" is simply an unsigned char.  It is a typedef defined 
 * in common/common.hpp.
 * @param workUnitData The work unit data to process.
 * @return Returns the calculated result data.
 */
vector<byte> ScienceDataProcessor::run(vector<byte>& workUnitData) {
	// Check the client for any saved checkpoints, if your science application uses checkpointing.
	vector<byte> checkpoint = ScienceApplicationClient::instance()->getCheckpoint();
	
	// We know a checkpoint was retrieved if checkpoint.size() is greater than zero.
	if (checkpoint.size() > 0) {
		cout << "Science Application: Recovered a checkpoint." << endl;
		// TODO: Interpret the checkpoint data.
		// TODO: Use it to initialize the science algorithm.
	}

	// Initialize a low-priority compute thread in which 
	// the science algorithm will be executed.
	pthread_attr_t threadAttrs;
	pthread_attr_init(&threadAttrs);
	struct sched_param threadSchedParam;
	threadSchedParam.sched_priority = sched_get_priority_min(SCHED_FIFO);
	pthread_attr_setschedparam(&threadAttrs, &threadSchedParam);

	// TODO: threadParam should be something meaningful, like a range to check
	// or even the work unit itself.
	void* threadParam = NULL;

	// Start scienceAlgorithm() in the compute thread.
	pthread_create(&computeThread, &threadAttrs, scienceAlgorithm, (void*)threadParam);
	
	// Then, wait for the thread to complete.
	void* computeThreadResult;
	pthread_join(computeThread, &computeThreadResult);

	// TODO: It will be necessary to cast the void*
	// into the actual type that was returned.
	// This can be done with a static_cast.

	// TODO: Convert the result returned by the science algorithm into a byte vector.
	vector<byte> resultData;

	// TODO: Delete computeThreadResult if it was allocated on the heap.

	// Return the result.
	return resultData;
}

/**
 * This is the science algorithm.
 * @param threadParam Any parameters that needed to be passed to the science algorithm.
 * @return Returns the result for the current work unit.
 */
void* ScienceDataProcessor::scienceAlgorithm(void* threadParam) {
	// TODO: Cast the threadParam to its actual type.

	cout << "Science Application: Running science algorithm." << endl;

	// TODO: Run the science algorithm.
		
	// When the algorithm is done, there should be some result, 
	// either a byte vector or some other result that the 
	// run() method can convert to a byte array.
	// The run() method actually returns the result 
	// to the project client.
		
	// You can save a checkpoint periodically so that if the 
	// science application were killed before finishing a work 
	// unit, it could resume from where it left off the last 
	// time.
	vector<byte> checkpointData;

	// Send the checkpoint to the project client.
	ScienceApplicationClient::instance()->saveCheckpoint(checkpointData);

	// TODO: Some result will need to be returned;
	void* result = NULL;
	return result;
}

} // end namespace prc
