#ifndef _SCI_DATA_PROC_HPP
#define _SCI_DATA_PROC_HPP

// System includes
#include <vector>

// XML-RPC includes
#include <xmlrpc-c/base.hpp>

using std::vector;

namespace prc {

/**
 * The ScienceDataProcess contains the project-specific science
 * algorithm.  This class should receive the byte array representing
 * a work unit, convert it to a meaningful form (if necessary), and 
 * start a low-priority compute thread to process the work unit data.
 * Finally, it should return a byte array representing the result 
 * for that work unit, which will be returned to the project server 
 * by the project client.
 */
class ScienceDataProcessor {
	private:
		pthread_t computeThread;
		static void* scienceAlgorithm(void*);
	public:
		/**
		 * This method will be called by the RPC handlers 
		 * after they pass in the work unit data through 
		 * the constructor call.  The run method should 
		 * execute the project-specific science algorithm
		 * on the work unit and produce a result in the 
		 * form of a byte array.  This byte array should 
		 * be passed back through the resultData parameter.
		 * The size of the resultData parameter should be 
		 * returned.
		 * @param resultData Should point to the calculated 
		 * result data after the science algorithm has completed.
		 * @return Returns the size of the resultData array.
		 */
		vector<byte> run(vector<byte>& resultData);
};

} // end namespace prc

#endif // _SCI_DATA_PROC
