// System includes
#include <iostream>
#include <cstring>
#include <vector>

// Local includes
#include "common/common.hpp"
#include "result_validator.hpp"

using std::cout;
using std::endl;
using std::string;
using std::vector;

namespace prc {

/**
 * Determines whether a single result was valid.
 * NOTE: Result is a struct defined in validator_client.hpp.
 * @param r The result to validate.
 * @return Returns true if the result is valid, false otherwise.
 */
bool ResultValidator::validateSingleResult(Result& r) {
	bool isValid = true;
	cout << "Result Validator: Validating Result " << r.resultID << endl;

	// TODO: Inspect the result data to determine whether it is valid.
	vector<byte>* resultData = &(r.data);

	// If necessary, it is possible to get the work unit from which this result was computed.
	// vector<byte> workUnitData = ValidatorClient::instance()->getAssociatedWorkUnit(r.resultID);
	
	return isValid;
}

/**
 * Selects a canonical result from a set of results.
 * NOTE: Result is a struct defined in validator_client.hpp.
 * @param results The set of valid results for a work unit.
 * @return Returns the result ID of the canonical result that was selected.
 */
string ResultValidator::selectCanonicalResult(vector<Result>& results) {
	// Initialize the canonical result to the first result in the Vector
	// so that we are sure to return some valid ID.
	string canonicalResult = results[0].resultID;

	// If necessary, it is possible to get the work unit from which this result was computed.
	// The work unit will be the same for all results in the Vector.
	// vector<byte> workUnitData = ValidatorClient::instance()->getAssociatedWorkUnit(results[0].resultID);

	// Loop through, examining the results, and choose one to be the canonical result. 
	for (unsigned int ii = 0; ii < results.size(); ii++) {
		// Examine the result data.
		vector<byte>* resultData = &(results[ii].data);
			
		// If this result should be the canonical result, set canonicalResult to this result's ID.
		canonicalResult = results[ii].resultID;
	}

	// Return the ID of the canonical result.
	cout << "Result Validator: Selecting canonical result " << canonicalResult << endl;
	return canonicalResult;
}

/**
 * Determines whether a spot-check was valid.
* NOTE: SpotCheck is a struct defined in validator_client.hpp.
 * @param check The spot-check to validate.
 * @return Returns true if the spot-check is valid, false otherwise.
 */
bool ResultValidator::validateSpotCheck(SpotCheck& check) {
	bool isValid = true;
		
	// The client data is the result data computed by a volunteer's computer.
	vector<byte>* clientData = &(check.clientData);
		
	// The accepted data is the result data that was computed by the project server.
	vector<byte>* acceptedData = &(check.acceptedData);
		
	// If necessary, it is possible to get the work unit from which this spot-check was computed.
	// vector<byte> workUnitData = ValidatorClient::instance()->getAssociatedWorkUnit(check.resultID);
	
	// Compare the two byte arrays using some algorithm, and determine 
	// whether the spot-check was valid. 

	return isValid;
}

} // end namespace prc
