package edu.wpi.prc.templates.java.science;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;

import edu.wpi.prc.templates.java.common.Server;

/**
 * The ScienceApplication class initializes and starts the 
 * XML-RPC server.  It handles requests from the project client
 * or project server, computing results for work units.
 * @author James Baldassari
 */
public class ScienceApplication {
	private static final int SLEEP_TIME = 2000;
	private static final int SCI_APP_PORT = 2082;	// Port on which the science application's XML-RPC server is listening.
	private static final String STDERR_LOG_FILE = "log" + File.separator + "sciapp.err.log";
	private static final String STDOUT_LOG_FILE = "log" + File.separator + "sciapp.out.log";
	Server server = null;
	
	/**
	 * Default constructor.
	 * Initializes the XML-RPC server.
	 */
	public ScienceApplication() {
		server = new Server(SCI_APP_PORT);
		server.registerRPCHandler("sciapp", new RPCHandler(this));
	}
	
	/**
	 * @return Returns true if the science application is running, false otherwise.
	 */
	public boolean isRunning() {
		return server.isRunning();
	}
	
	/**
	 * Starts the XML-RPC server.
	 */
	public void start() {
		server.start();
	}
	
	/**
	 * Shuts down the XML-RPC server, which will cause the main to exit.
	 * @return Always returns true.
	 */
	public boolean shutdown() {
		return server.shutdown();
	}
	
	/**
	 * Initializes and starts the science application.
	 * @param args Command-line arguments.
	 */
	public static void main(String[] args) {
		// Redirect standard output and error output to a file.
		FileOutputStream errOS = null;
		FileOutputStream stdOS = null;
		try {
			errOS = new FileOutputStream(STDERR_LOG_FILE);
			stdOS = new FileOutputStream(STDOUT_LOG_FILE);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		PrintStream errPS = new PrintStream(errOS);
		PrintStream stdPS = new PrintStream(stdOS);
		System.setErr(errPS);
		System.setOut(stdPS);
		
		ScienceApplication sciApp = new ScienceApplication();
		
		// Start the science application.
		sciApp.start();
		
		System.out.println("Science Application Ready.");
		
		// The XML-RPC server is in a separate thread, so just sleep until termination.
		while (sciApp.isRunning() == true) {
			try {
				Thread.sleep(SLEEP_TIME);
			} catch (InterruptedException e) {

			}
		}
		
		System.out.println("Science Application Halted.");
	}
}
