package edu.wpi.prc.templates.java.science;

/**
 * The ScienceDataProcessor contains all of the project-specific methods 
 * required to compute the results for work units.  This class is used 
 * by the ComputeThread.
 * @author James Baldassari
 */
public class ScienceDataProcessor {
	private byte[] workUnitData = null;
	
	/**
	 * Constructor that takes a byte array representing the 
	 * work unit data to be processed.
	 * @param workUnitData The work unit data.
	 */
	public ScienceDataProcessor(byte[] workUnitData) {
		this.workUnitData = workUnitData;
	}
	
	/**
	 * This method checks for previously saved checkpoints, then runs the 
	 * science algorithm.  It then converts the result into a byte array,
	 * which will be returned to the server.
	 * @return Returns a byte array representing the computed result.
	 */
	public byte[] computeResult() {		
		// Extract any necessary information from the work unit, which is
		// stored in the workUnitData member variable.
		
		// Check the client for any saved checkpoints, if your science application uses checkpointing.
		byte[] checkpoint = ScienceApplicationClient.instance().getCheckpoint();
		
		// We know a checkpoint was retrieved if checkpoint.length is greater than zero.
		if (checkpoint.length > 0) {
			System.out.println("Science Application: Recovered a checkpoint.");
			// TODO: Interpret the checkpoint data.
			// TODO: Use it to initialize the science algorithm.
		}
		
		// Run the science algorithm.
		scienceAlgorithm();
		
		// Convert the result computed by the science algorithm into
		// a byte array.  The byte array should be allocated with enough
		// space to store the result, not with size zero as done below.
		byte[] resultData = new byte[0];
		
		// Return the computed result.
		return resultData;
	}
	
	/**
	 * This is the science algorithm.
	 */
	private void scienceAlgorithm() {	
		System.out.println("Science Application: Running science algorithm.");
		
		// TODO: Run the science algorithm.  This method can be modified to 
		// take parameters and return some value, or member variables 
		// can be used.
		
		// When the algorithm is done, there should be some result, 
		// either a byte array or some other result that the 
		// computeResult() method can convert to a byte array.
		// The result can be saved in a member variable, or this 
		// method can be modified to have a return value.
		// The computeResult() method actually returns the result 
		// to the project client.
		
		// You can save a checkpoint periodically so that if the 
		// science application were killed before finishing a work 
		// unit, it could resume from where it left off the last 
		// time.  The byte array should be allocated appropriately 
		// depending on the checkpoint contents, not with zero 
		// length as it is below.
		byte[] checkpoint = new byte[0];
		
		// Send the checkpoint to the project client.
		ScienceApplicationClient.instance().saveCheckpoint(checkpoint);
	}
}
