package edu.wpi.prc.project;

/**
 * Indicates what action to take when a computer fails a spot check.
 * NotifyAdmin means that the project administrator will be notified that the user failed a spot check.
 * BanUser means that the user will be permanently banned from submitting results and requesting work units.
 * PurgeResults means that the user will be permanently banned and all results the user has ever returned will be deleted.
 * @author James Baldassari
 */
public enum SpotCheckFailureAction {
	Log("Log"),						// Log the spot check failure.
	BanUser("BanUser"),				// Log the failure and permanently ban the user from submitting results and requesting work units.
	PurgeResults("PurgeResults");	// Log the failure, ban the user, and delete all results the user has ever returned.
	
	private final String actionStr;
	
	SpotCheckFailureAction(String actionStr) {
		this.actionStr = actionStr;
	}
	
	public String toString() {
		return actionStr;
	}
}
