/**
 * 
 */
package edu.wpi.prc.server.validator;

import java.util.Vector;

import edu.wpi.prc.persistence.SysLogger;
import edu.wpi.prc.server.EmptyQueueException;
import edu.wpi.prc.server.PRCServer;

/**
 * This is the default validator.  It simply requests results 
 * from the server and immediately marks them as valid.  For 
 * canonical results, it selects the first reselt to be the 
 * canonical result.
 * @author James Baldassari
 */
public class DefaultValidatorThread extends Thread {
	private static final int RETRY_DELAY = 10000;		// Wait 10 seconds if the result queue is empty.
	private boolean isRunning = true;
	
	/**
	 * Requests results from the server and immediately validates them.
	 */
	public void run() {
		this.setName("Default Validator Thread");
		while (isRunning) {
			try {
				Vector result = PRCServer.instance().getRPCHandler().getResultForValidation();
				int resultType = ((Integer)result.get(0)).intValue();
				switch (resultType) {
				case 0:
					// Single result.  Just mark it as valid.
					SysLogger.println("DefaultResultValidatorThread: Validating individual result " + result.get(1) + ".");
					PRCServer.instance().getRPCHandler().markResultValid((String)result.get(1));
					break;
				case 1:
					// Multiple results.  Just select the first as the canonical result.
					SysLogger.println("DefaultResultValidatorThread: Selecting canonical result " + result.get(2) + ".");
					PRCServer.instance().getRPCHandler().selectCanonicalResult((String)result.get(2));
					break;
				case 2:
					// Spot check result.  The default validator should never be used with spot checks.
					SysLogger.println("DefaultResultValidatorThread: Validating spot check result " + result.get(1) + ".");
					PRCServer.instance().getRPCHandler().markResultValid((String)result.get(1));
					break;
				default:
					SysLogger.println("DefaultResultValidatorThread: Received unknown result type RPCHandler.getResultForValidation().");
				}
			} catch (EmptyQueueException e) {
				try {
					Thread.sleep(RETRY_DELAY);
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}
			}
		}
	}
	
	/**
	 * Shuts down the thread.
	 */
	public void shutdown() {
		isRunning = false;
	}
}
