package edu.wpi.prc.task;

/**
 * NonPersistentResult is a stripped-down version of the Result class.
 * The reason that there are two different classes is because Result 
 * depends on the Hibernate library.  The client does not need to use 
 * any Hibernate functionality, but if it were to use the Result 
 * class, the client would still need to have the Hibernate jar file, 
 * which is over 1.5MB in size.  The client distribution file should 
 * be as small as possible, so by using this version of the result 
 * class that does not depend on Hibernate, the client does not need 
 * to have the Hibernate jar file.  
 * @author James Baldassari
 */
public class NonPersistentResult extends Task {
	
	/**
	 * Default contstructor.
	 */
	public NonPersistentResult() {
		
	}
	
	/**
	 * Constructor that takes a task ID.
	 * @param taskID The taskID to set.
	 */
	public NonPersistentResult(String taskID) {
		super.setTaskID(taskID);
	}

	/**
	 * Only used in the persistent version of this class.
	 */
	@Override
	public void save() {
		
	}
	
	/**
	 * Only used in the persistent version of this class.
	 */
	@Override
	public void update() {
		
	}
	
	/**
	 * Only used in the persistent version of this class.
	 */
	@Override
	public void delete() {
		
	}
}
