package edu.wpi.prc.task.queue;

import java.util.Comparator;

/**
 * The PriorityComparator is used to compare the priorities
 * of two Task queues in the MultiPriorityQueue.
 * @author James Baldassari
 */
public class PriorityComparator implements Comparator<Integer> {

	/**
	 * Compares the priorities of two Task queues.
	 * @param firstPriority The priority of the first queue.
	 * @param secondPriority The priority of the second queue.
	 * @return Returns -1, 0, or 1 if the first queue has a higher, equal, or lower priority than the second
	 * because the queues are sorted in reverse order (highest priority first) in the MultiPriorityQueue.
	 */
	public int compare(Integer firstPriority, Integer secondPriority) {
		// Higher priorities should come before lower priorities, so 
		// return -1 if firstPriority > secondPriority.
		if (firstPriority.compareTo(secondPriority) > 0)
			return -1;
		else if (firstPriority.equals(secondPriority))
			return 0;
		else
			return 1;
	}
}
