package edu.wpi.prc.test;

import edu.wpi.prc.persistence.HibernateController;
import edu.wpi.prc.project.test.AllProjectTests;
import edu.wpi.prc.server.test.AllServerTests;
import edu.wpi.prc.task.test.AllTaskTests;
import edu.wpi.prc.user.test.AllUserTests;
import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * Runs all JUnit tests in the entire PRC system.
 * @author James Baldassari
 */
public class AllTests {
	/**
	 * Runs all tests in the PRC system.
	 * @param args Command-line arguments.
	 */
	public static void main(String[] args) {
		// For the unit tests configure Hibernate to use HSQLDB and the database data/prc.
		configureHibernate();
		System.out.println("Running all tests.");
		
		// Run the tests.
		junit.textui.TestRunner.run(suite());
	}
	
	/**
	 * Configures Hibernate for the unit tests.
	 */
	public static void configureHibernate() {
		HibernateController.instance().initializeForUnitTests();
		HibernateController.instance().exportSchema();
	}

	/**
	 * @return Returns a test containing all test suites in the PRC system.
	 */
	public static Test suite() {
		TestSuite suite = new TestSuite("Test for edu.wpi.prc.*.test");
		suite.addTest(AllTaskTests.suite());
		suite.addTest(AllUserTests.suite());
		suite.addTest(AllServerTests.suite());
		suite.addTest(AllProjectTests.suite());
		return suite;
	}

}
