package edu.wpi.prc.tools.builder.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JTextArea;

import com.jeta.forms.components.panel.FormPanel;

import edu.wpi.prc.project.Project;
import edu.wpi.prc.tools.builder.ProjectBuilder;

/**
 * Prompts user for project name and project password.
 * @author James Baldassari
 */
public class BasicConfigurationForm extends GUIForm {
	private static final long serialVersionUID = 1L;
	private GUIForm previousForm = null;
	private FormPanel currentForm = null;
	private GUIForm nextForm = null;
	private Project project = null;

	/**
	 * Constructor that takes a reference to the this form's parent form.
	 */
	public BasicConfigurationForm(GUIForm previousForm) {		
		this.previousForm = previousForm;
		currentForm = new FormPanel("edu/wpi/prc/tools/builder/gui/forms/BasicConfigurationForm.jfrm");
		setLayout(new BorderLayout());
		add(currentForm, BorderLayout.CENTER);
		this.setTitle("PRC Project Builder: Step 2 of 7");

		// Add action listeners.
		currentForm.getButton(BasicConfigurationComponents.ID_PREVIOUS_BUTTON).addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				selectActionPreviousClick();
			}
		});
		
		currentForm.getButton(BasicConfigurationComponents.ID_NEXT_BUTTON).addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				selectActionNextClick();
			}
		});
		
		((JTextArea)currentForm.getComponentByName(BasicConfigurationComponents.ID_HELP_TEXT_AREA)).setCaretPosition(0);
		this.setSize(430, 373);
		ProjectBuilder.addWindowCloseListener(this);
	}
	
	/**
	 * Populates all fields using the project data when this form is set to visible.
	 */
	public void setVisible(boolean isVisible) {
		if (isVisible == true) {
			populateFields();
		}
		super.setVisible(isVisible);
	}
	
	/**
	 * Reads information from the project and sets the fields in the form accordingly.
	 */
	private void populateFields() {
		// Get the project from the ProjectBuilder.
		project = ProjectBuilder.instance().getProject();
		
		// Populate the fields.
		currentForm.getTextField(BasicConfigurationComponents.ID_PROJECT_NAME_FIELD).setText(project.getName());
		currentForm.getTextField(BasicConfigurationComponents.ID_PROJECT_PASSWORD_FIELD).setText(project.getPassword());
		currentForm.getTextField(BasicConfigurationComponents.ID_CONFIRM_PASSWORD_FIELD).setText(project.getPassword());
	}
	
	/**
	 * Updates the project properties using the values in the form fields. 
	 */
	private void updateProject() {
		project.setName(currentForm.getTextField(BasicConfigurationComponents.ID_PROJECT_NAME_FIELD).getText());
		project.setPassword(currentForm.getTextField(BasicConfigurationComponents.ID_PROJECT_PASSWORD_FIELD).getText());
	}
	
	/**
	 * Checks whether all input fields are correct and displays a dialog message if there are any errors.
	 * @return Returns true if all input fields are valid, false otherwise.
	 */
	private boolean validateInput() {
		boolean isValid = true;
		if (project.getName().length() == 0) {
			isValid = false;
			ProjectBuilder.instance().showErrorDialog(this, "The project name must be specified.");
		}
		else if (currentForm.getTextField(BasicConfigurationComponents.ID_PROJECT_PASSWORD_FIELD).getText().equals(
				currentForm.getTextField(BasicConfigurationComponents.ID_CONFIRM_PASSWORD_FIELD).getText()) == false) {
			isValid = false;
			String message = "The two passwords you have entered are not the same.  Please enter\n";
			message += "your password again in both fields to verify that it is correct.";
			ProjectBuilder.instance().showErrorDialog(this, message);
		}
		else if (project.getPassword().length() < 6) {
			isValid = false;
			ProjectBuilder.instance().showErrorDialog(this, "Your project password must be at least six characters long.");
		}
		return isValid;
	}
	
	/**
	 * Controls the actions that occur when the previous button is clicked.
	 */
	private void selectActionPreviousClick() {
		updateProject();
		ProjectBuilder.instance().setProject(project);
		this.setVisible(false);
		ProjectBuilder.setRelativePosition(previousForm, this);
		previousForm.setVisible(true);
	}
	
	/**
	 * Controls the actions that occur when the next button is clicked.
	 */
	private void selectActionNextClick() {
		updateProject();
		if (validateInput() == true) {
			ProjectBuilder.instance().setProject(project);
			if (nextForm == null)
				nextForm = new DatabaseConfigurationForm(this);
			ProjectBuilder.setRelativePosition(nextForm, this);
			nextForm.setVisible(true);
			this.setVisible(false);
		}
	}
}
