package edu.wpi.prc.tools.builder.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JTextArea;

import com.jeta.forms.components.panel.FormPanel;

import edu.wpi.prc.persistence.DatabaseType;
import edu.wpi.prc.project.Project;
import edu.wpi.prc.tools.builder.ProjectBuilder;

/**
 * @author James Baldassari
 *
 */
public class DatabaseConfigurationForm extends GUIForm {
	private static final long serialVersionUID = 1L;
	private GUIForm previousForm = null;
	private FormPanel currentForm = null;
	private GUIForm nextForm = null;
	private Project project = null;
	private DatabaseType originalDbType = null;
	private String originalDbURI = null;
	
	/**
	 * Constructor that takes a reference to the this form's parent form.
	 */
	DatabaseConfigurationForm(GUIForm previousForm) {
		this.previousForm = previousForm;
		currentForm = new FormPanel("edu/wpi/prc/tools/builder/gui/forms/DatabaseConfigurationForm.jfrm");
		setLayout(new BorderLayout());
		add(currentForm, BorderLayout.CENTER);
		this.setTitle("PRC Project Builder: Step 3 of 7");
		
		// If modifying an existing project, cannot change the DB type or address.
		originalDbType = ProjectBuilder.instance().getProject().getDbType();
		originalDbURI = ProjectBuilder.instance().getProject().getDbURI();
		
		// Add action listeners.
		
		currentForm.getComboBox(DatabaseConfigurationComponents.ID_DATABASE_TYPE_COMBO).addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				selectDatabaseType(event);
			}
		});
		
		currentForm.getButton(DatabaseConfigurationComponents.ID_PREVIOUS_BUTTON).addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				selectActionPreviousClick();
			}
		});
		
		currentForm.getButton(DatabaseConfigurationComponents.ID_NEXT_BUTTON).addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				selectActionNextClick();
			}
		});
		
		((JTextArea)currentForm.getComponentByName(DatabaseConfigurationComponents.ID_HELP_TEXT_AREA)).setCaretPosition(0);
		this.setSize(430, 403);
		ProjectBuilder.addWindowCloseListener(this);
	}
	
	/**
	 * Populates all fields using the project data when this form is set to visible.
	 */
	public void setVisible(boolean isVisible) {
		if (isVisible == true) {
			populateFields();
		}
		super.setVisible(isVisible);
	}
	
	/**
	 * Reads information from the project and sets the fields in the form accordingly.
	 */
	private void populateFields() {
		// Get the project from the ProjectBuilder.
		project = ProjectBuilder.instance().getProject();
		
		// Populate the fields
		Vector<DatabaseType> databaseTypeItems = new Vector<DatabaseType>(5);
		databaseTypeItems.add(DatabaseType.HSQLDB);
		databaseTypeItems.add(DatabaseType.MySQL);
		databaseTypeItems.add(DatabaseType.PostgreSQL);
		databaseTypeItems.add(DatabaseType.Oracle);
		databaseTypeItems.add(DatabaseType.MSSQLServer);
		currentForm.getComboBox(DatabaseConfigurationComponents.ID_DATABASE_TYPE_COMBO).setModel(new DefaultComboBoxModel(databaseTypeItems));
		currentForm.getComboBox(DatabaseConfigurationComponents.ID_DATABASE_TYPE_COMBO).setEnabled(true);
		currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setEnabled(true);
		
		String dbURI = project.getDbURI();
		if (dbURI != null)
			currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setText(project.getDbURI());
		else
			currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setText("data" + File.separator + "prcdb");
		currentForm.getTextField(DatabaseConfigurationComponents.ID_USER_NAME_FIELD).setText(project.getDbUserName());
		currentForm.getTextField(DatabaseConfigurationComponents.ID_PASSWORD_FIELD).setText(project.getDbPassword());
		
		DatabaseType dbType = project.getDbType();
		if (dbType != null)
			currentForm.getComboBox(DatabaseConfigurationComponents.ID_DATABASE_TYPE_COMBO).setSelectedItem(dbType);
		else
			currentForm.getComboBox(DatabaseConfigurationComponents.ID_DATABASE_TYPE_COMBO).setSelectedItem(DatabaseType.HSQLDB);
	}
	
	/**
	 * This method is called when the user changes the database type that is selected.
	 * @param event The event that triggered this action listener.
	 */
	private void selectDatabaseType(ActionEvent event) {
		String dbURI = project.getDbURI();
		
		if (currentForm.getComboBox(DatabaseConfigurationComponents.ID_DATABASE_TYPE_COMBO).getSelectedItem().equals(DatabaseType.HSQLDB)) {
			if (dbURI != null)
				currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setText(project.getDbURI());
			else
				currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setText("data" + File.separator + "prcdb");
			
			currentForm.getTextField(DatabaseConfigurationComponents.ID_USER_NAME_FIELD).setText("sa");
			currentForm.getTextField(DatabaseConfigurationComponents.ID_PASSWORD_FIELD).setText("");
			currentForm.getLabel(DatabaseConfigurationComponents.ID_USER_NAME_LABEL).setEnabled(false);
			currentForm.getTextField(DatabaseConfigurationComponents.ID_USER_NAME_FIELD).setEnabled(false);
			currentForm.getLabel(DatabaseConfigurationComponents.ID_PASSWORD_LABEL).setEnabled(false);
			currentForm.getTextField(DatabaseConfigurationComponents.ID_PASSWORD_FIELD).setEnabled(false);	
		}
		
		else {
			if (dbURI == null)
				currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setText("");
			else if (dbURI.startsWith("data/"))
				currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setText("");
			else
				currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setText(project.getDbURI());
			
			String dbUserName = project.getDbUserName();
			if (dbUserName == null)
				currentForm.getTextField(DatabaseConfigurationComponents.ID_USER_NAME_FIELD).setText("");
			else if (dbUserName.equals("sa"))
				currentForm.getTextField(DatabaseConfigurationComponents.ID_USER_NAME_FIELD).setText("");
			else
				currentForm.getTextField(DatabaseConfigurationComponents.ID_USER_NAME_FIELD).setText(dbUserName);
				
			
			String dbPassword = project.getDbPassword();
			if (dbPassword != null)
				currentForm.getTextField(DatabaseConfigurationComponents.ID_PASSWORD_FIELD).setText(dbPassword);
			else
				currentForm.getTextField(DatabaseConfigurationComponents.ID_PASSWORD_FIELD).setText("");
			
			currentForm.getLabel(DatabaseConfigurationComponents.ID_USER_NAME_LABEL).setEnabled(true);
			currentForm.getTextField(DatabaseConfigurationComponents.ID_USER_NAME_FIELD).setEnabled(true);
			currentForm.getLabel(DatabaseConfigurationComponents.ID_PASSWORD_LABEL).setEnabled(true);
			currentForm.getTextField(DatabaseConfigurationComponents.ID_PASSWORD_FIELD).setEnabled(true);
		}
	}
	
	/**
	 * Updates the project properties using the values in the form fields. 
	 */
	private void updateProject() {
		DatabaseType selectedDB = Enum.valueOf(DatabaseType.class, currentForm.getComboBox(DatabaseConfigurationComponents.ID_DATABASE_TYPE_COMBO).getSelectedItem().toString());
		String databaseURI = currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).getText();
		String userName = currentForm.getTextField(DatabaseConfigurationComponents.ID_USER_NAME_FIELD).getText();
		String password = currentForm.getTextField(DatabaseConfigurationComponents.ID_PASSWORD_FIELD).getText();
		
		if (password.equals(""))
			password = null;
		
		project.setDbType(selectedDB);
		project.setDbURI(databaseURI);
		project.setDbUserName(userName);
		project.setDbPassword(password);
	}
	
	/**
	 * Checks whether all input fields are correct and displays a dialog message if there are any errors.
	 * @return Returns true if all input fields are valid, false otherwise.
	 */
	private boolean validateInput() {
		boolean isValid = true;
		if (project.getDbURI().length() == 0) {
			isValid = false;
			ProjectBuilder.instance().showErrorDialog(this, "The database address must be specified.");
		}
		else if ((project.getDbType() != DatabaseType.HSQLDB) && (project.getDbUserName().length() == 0)) {
			isValid = false;
			ProjectBuilder.instance().showErrorDialog(this, "The database user name must be specified.");
		} else if (ProjectBuilder.instance().isNewProject() == false) {
			if ((project.getDbType() != originalDbType) || !project.getDbURI().equals(originalDbURI)) {
				isValid = false;
				ProjectBuilder.instance().showErrorDialog(this, "The database type and address of an existing project cannot be changed.");
				currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setText(originalDbURI);
				currentForm.getComboBox(DatabaseConfigurationComponents.ID_DATABASE_TYPE_COMBO).setSelectedItem(originalDbType);
				currentForm.getTextField(DatabaseConfigurationComponents.ID_DATABASE_ADDRESS_FIELD).setEnabled(false);
				currentForm.getComboBox(DatabaseConfigurationComponents.ID_DATABASE_TYPE_COMBO).setEnabled(false);
			}
		}
		return isValid;
	}
	
	/**
	 * Controls the actions that occur when the previous button is clicked.
	 */
	private void selectActionPreviousClick() {
		updateProject();
		ProjectBuilder.instance().setProject(project);
		this.setVisible(false);
		ProjectBuilder.setRelativePosition(previousForm, this);
		previousForm.setVisible(true);
	}
	
	/**
	 * Controls the actions that occur when the next button is clicked.
	 */
	private void selectActionNextClick() {
		updateProject();
		if (validateInput() == true) {
			ProjectBuilder.instance().setProject(project);
			if (nextForm == null)
				nextForm = new NetworkConfigurationForm(this);
			nextForm.setLocation(this.getLocation());
			nextForm.setVisible(true);
			this.setVisible(false);
		}
	}
}
