package edu.wpi.prc.tools.builder.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JTextArea;

import org.hibernate.HibernateException;

import com.jeta.forms.components.panel.FormPanel;
import edu.wpi.prc.project.Project;
import edu.wpi.prc.tools.builder.ProjectBuilder;

/**
 * Asks user whether he/she is creating a new project or modifying an existing project.
 * @author James Baldassari
 */
public class SelectActionForm extends DatabaseInitializerForm {
	private static final long serialVersionUID = 1L;
	private FormPanel currentForm = null;
	private GUIForm nextForm = null;
	private Project project = null;
	private ConnectionFormThread connectionFormThread = null;
	boolean isNewProject;

	/**
	 * Constructor that takes a reference to the this form's parent form.
	 */
	public SelectActionForm() {
		currentForm = new FormPanel("edu/wpi/prc/tools/builder/gui/forms/SelectActionForm.jfrm");
		setLayout(new BorderLayout());
		add(currentForm, BorderLayout.CENTER);
		this.setTitle("PRC Project Builder: Step 1 of 7");
		ProjectBuilder.centerComponent(this);

		// Add action listeners.
		currentForm.getButton(SelectActionComponents.ID_NEXT_BUTTON).addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				selectActionNextClick();
			}
		});
		
		((JTextArea)currentForm.getComponentByName(SelectActionComponents.ID_HELP_TEXT_AREA)).setCaretPosition(0);
		this.setSize(430, 333);
		ProjectBuilder.addWindowCloseListener(this);
	}
	
	protected void completeDatabaseConnection() {
		boolean dbConnectionSuccess = true;
		if ((isNewProject == false) && (connectionFormThread != null)) {
			try {
				project = connectionFormThread.getProject();
			} catch (HibernateException e) {
				dbConnectionSuccess = false;
				String message = "The Database connection failed.  If you are using an HSQLDB\n";
				message += "database, verify that the database files exist in the location\n";
				message += "specified in your project configuration file.  If you are not\n";
				message += "using an HSQLDB database, verify that your database is\n";
				message += "configured correctly, that it is accepting connections, and\n";
				message += "that this computer can connect to the computer hosting\n";
				message += "the database.";
				ProjectBuilder.instance().showErrorDialog(this, message);
			}
		}
		
		if (project == null)
			project = Project.createProject();
		
		if (dbConnectionSuccess == true) {
			ProjectBuilder.instance().setProject(project);
			ProjectBuilder.instance().setNewProject(isNewProject);
			
			if (nextForm == null)
				nextForm = new BasicConfigurationForm(this);
			
			if ((isNewProject == false) && (connectionFormThread != null))
				ProjectBuilder.setRelativePosition(nextForm, connectionFormThread.getForm());
			else
				ProjectBuilder.setRelativePosition(nextForm, this);
			
			nextForm.setVisible(true);
			this.setVisible(false);
		}
		else
			this.setVisible(true);
		
		if (connectionFormThread != null)
			connectionFormThread.cleanup();
	}
	
	/**
	 * Controls the actions that occur when the next button is clicked.
	 */
	private void selectActionNextClick() {
		isNewProject = currentForm.getRadioButton(SelectActionComponents.ID_NEW_PROJECT_RADIO).isSelected();
		if (isNewProject == true) {
			project = Project.createProject();
			completeDatabaseConnection();
		}
		else {
			connectionFormThread = new ConnectionFormThread(this);
			ProjectBuilder.setRelativePosition(connectionFormThread.getForm(), this);
			connectionFormThread.start();
			// When this thread has finished establishing the database connection, it will call completeDatabaseConnection().
			
			this.setVisible(false);
		}
	}
}