package edu.wpi.prc.user.test;

import java.util.Date;

import edu.wpi.prc.persistence.HibernateController;
import edu.wpi.prc.user.Volunteer;
import junit.framework.TestCase;

/**
 * Tests Volunteer persistence.
 * @author James Baldassari
 */
public class VolunteerTest extends TestCase {
	private Volunteer v = new Volunteer();
	
	public VolunteerTest(String name) {
		super(name);
		
		HibernateController.instance().initializeForUnitTests();
		
		v.setUserID("jdb");
		v.setEMailAddress("jdb@cs.wpi.edu");
		v.setScore(1);
		v.setJoinDate(new Date());
		v.setLastResultDate(new Date());
	}

	protected void setUp() throws Exception {
		super.setUp();
		HibernateController.instance().exportSchema();
	}
	
	/**
	 * Tests saving a Volunteer to the database and retrieving it from the database.
	 */
	public void testSave() {
		// Save the Volunteer to the database
		v.save();
		
		// Create a new Volunteer with the same information.
		Volunteer v2 = new Volunteer("jdb");
		v2.setEMailAddress("jdb@cs.wpi.edu");
		v2.setScore(1);
		v2.setJoinDate(v.getJoinDate());
		v2.setLastResultDate(v.getLastResultDate());
		
		// Find the Volunteer in the database.
		Volunteer[] matches = v2.find();
		
		// Verify that it contains the correct information.
		assertTrue(matches.length == 1);
		assertTrue(matches[0].getUserID().equals("jdb"));
		assertTrue(matches[0].getEMailAddress().equals("jdb@cs.wpi.edu"));
		assertTrue(matches[0].getScore() == 1);
	}
	
	/**
	 * Tests the increaseScore function.
	 */
	public void testIncreaseScore() {
		Volunteer v1 = new Volunteer();
		v1.setUserID("jdb");
		v1.setEMailAddress("jdb@cs.wpi.edu");
		v1.setJoinDate(new Date());
		v1.save();
		
		Volunteer search = new Volunteer();
		search.setUserID("jdb");
		Volunteer match = search.find()[0];
		assertTrue(match.getScore() == 0);
		
		match.increaseScore(10);
		match.update();
		
		Volunteer match2 = search.find()[0];
		assertTrue(match2.getScore() == 10);
	}
	
	/**
	 * Test modifying a Volunteer.
	 */
	public void testUpdate() {
		// Save the Volunteer to the database
		v.save();
		
		// Find the Volunteer in the database.
		Volunteer[] matches = v.find();
		
		// Modify the Volunteer.
		matches[0].setEMailAddress("jbaldassari@gmail.com");
		
		// Update the volunteer in the database.
		matches[0].update();
		
		// Get all Volunteers (should only be 1)
		matches = null;
		matches = Volunteer.findAll();
		
		// Verify that it contains the correct information.
		assertTrue(matches.length == 1);
		assertTrue(matches[0].getEMailAddress().equals("jbaldassari@gmail.com"));
	}
	
	/**
	 * Test deleting a Volunteer from the database.
	 */
	public void testDelete() {
		// Save the Volunteer to the database
		v.save();
		
		// Verify that there is exactly one Volunteer in the database.
		Volunteer[] matches = Volunteer.findAll();
		assertTrue(matches.length == 1);
		
		// Delete the Volunteer from the database.
		v.delete();
		
		// Verify that there are no long any Volunteers in the database.
		matches = Volunteer.findAll();
		assertTrue(matches.length == 0);
	}

}
